/*
 * Copyright 2010 James Pether Sörling
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *	$Id: RiksdagenApi.java 4642 2010-09-24 10:45:33Z pether $
 *  $HeadURL: https://cia.svn.sourceforge.net/svnroot/cia/tags/service.external.riksdagen-1.0.0-beta-1.1/src/main/java/com/hack23/cia/service/external/riksdagen/api/RiksdagenApi.java $
*/
package com.hack23.cia.service.external.riksdagen.api;

import java.util.List;

import com.hack23.cia.model.external.riksdagen.documentcontent.impl.DocumentContentData;
import com.hack23.cia.model.external.riksdagen.dokumentlista.impl.DocumentElement;
import com.hack23.cia.model.external.riksdagen.dokumentstatus.impl.DocumentStatusContainer;
import com.hack23.cia.model.external.riksdagen.dokumentstatus.impl.DocumentType;
import com.hack23.cia.model.external.riksdagen.person.impl.PersonData;
import com.hack23.cia.model.external.riksdagen.person.impl.RoleStatus;
import com.hack23.cia.model.external.riksdagen.personlista.impl.PersonContainerElement;
import com.hack23.cia.model.external.riksdagen.utskottsforslag.impl.CommitteeProposalComponentData;
import com.hack23.cia.model.external.riksdagen.votering.impl.VoteData;
import com.hack23.cia.model.external.riksdagen.voteringlista.impl.BallotDocumentElement;
import com.hack23.cia.service.external.common.api.ProcessDataStrategy;


/**
 * The Interface RiksdagenApi.
 */
public interface RiksdagenApi {

	/**
	 * The Enum DocumentTypeNames.
	 */
	public enum DocumentTypeNames {

		/** The ACCOUN t_ an d_ report. */
		ACCOUNT_AND_REPORT("frsrdg","Framställning och redogörelse"),

		/** The AGENDA. */
		AGENDA("f-lista","Föredragningslista"),

		/** The ANSWE r_ t o_ question. */
		ANSWER_TO_QUESTION("frs","Svar på skriftlig fråga"),

		/** The ATTACHEMENT. */
		ATTACHEMENT("bilaga","Bilaga"),

		/** The BILL. */
		BILL("mot","Motion"),

		/** The COMMITTE e_ document. */
		COMMITTEE_DOCUMENT("utskottsdokument","Utskottsdokument"),

		/** The COMMITTE e_ report. */
		COMMITTEE_REPORT("bet","Betänkande"),

		/** The COMMITTE e_ story. */
		COMMITTEE_STORY("komm","Kommittéberättelser"),

		/** The COMMUNICATION. */
		COMMUNICATION("skr","Skrivelse"),

		/** The DEPARTMEN t_ serie. */
		DEPARTMENT_SERIE("ds","Departementsserien (Ds)"),

		/** The DIRECTIVE. */
		DIRECTIVE("dir","Direktiv"),

		/** The E u_ committee. */
		EU_COMMITTEE("eundok","EU-nämndsdokument"),

		/** The E u_ committe e_ attachement. */
		EU_COMMITTEE_ATTACHEMENT("eunbil","Bilaga till EU-nämndsdokument"),

		/** The E u_ committe e_ notes. */
		EU_COMMITTEE_NOTES("eunprot","EU-nämndens stenografiska uppteckningar"),

		/** The E u_ committe e_ summon s_ an d_ agenda. */
		EU_COMMITTEE_SUMMONS_AND_AGENDA("kf-lista","EU-nämndens kallelser och föredragningslistor"),

		/** The FAC t_ memorandum. */
		FACT_MEMORANDUM("fpm","Faktapromemoria"),

		/** The INTERPELLATION. */
		INTERPELLATION("ip","Interpellation"),

		/** The MINISTE r_ memorandum. */
		MINISTER_MEMORANDUM("minråd","Ministerrådspromemoria"),

		/** The PARLIAMEN t_ communication. */
		PARLIAMENT_COMMUNICATION("rskr","Riksdagsskrivelse"),

		/** The PARLIAMEN t_ report. */
		PARLIAMENT_REPORT("rfr","Rapport från riksdagen"),

		/** The Proposition. */
		Proposition("prop","Propositioner"),

		/** The PROTOCOL. */
		PROTOCOL("prot","Protokoll"),

		/** The PUBLI c_ investigation. */
		PUBLIC_INVESTIGATION("sou","Statens offentliga utredningar (SOU)"),

		/** The QUESTION. */
		QUESTION("fr","Skriftlig fråga"),

		/** The SPEAKE r_ list. */
		SPEAKER_LIST("t-lista","Talarlista"),

		/** The STATEMENT. */
		STATEMENT("yttr","Yttrande");

		/** The description. */
		private final String description;

		/** The short code. */
		private final String shortCode;

		/**
		 * Instantiates a new document type names.
		 *
		 * @param shortCode
		 *            the short code
		 * @param description
		 *            the description
		 */
		DocumentTypeNames(final String shortCode,final String description) {
			this.shortCode = shortCode;
			this.description = description;
		}

		/**
		 * Gets the description.
		 *
		 * @return the description
		 */
		public String getDescription() {
			return this.description;
		}

		/**
		 * Gets the short code.
		 *
		 * @return the short code
		 */
		public String getShortCode() {
			return this.shortCode;
		}
	}

	/** The Constant closed. */
	public static final RoleStatus[] closed = {RoleStatus.AVLIDEN_2000_04_28,RoleStatus.AVLIDEN_2003_09_11};

	/** The Constant RIKSDAGEN_JAVA_SIMPLE_DATE_FORMAT. */
	public static final String RIKSDAGEN_JAVA_SIMPLE_DATE_FORMAT="yyyy-MM-dd HH:mm:ss";

	/** The Constant RIKSDAGEN_SQL_STR_TO_DATE_FORMAT. */
	public static final String RIKSDAGEN_SQL_STR_TO_DATE_FORMAT="%Y-%m-%d %H:%i:%S";

	/**
	 * Gets the ballot.
	 *
	 * @param id
	 *            the id
	 * @return the ballot
	 * @throws Exception
	 *             the exception
	 */
	List<VoteData> getBallot(String id) throws Exception;

	/**
	 * Gets the committee proposal.
	 *
	 * @param id
	 *            the id
	 * @return the committee proposal
	 * @throws Exception
	 *             the exception
	 */
	CommitteeProposalComponentData getCommitteeProposal(String id) throws Exception;

	/**
	 * Gets the document content.
	 *
	 * @param id
	 *            the id
	 * @return the document content
	 * @throws Exception
	 *             the exception
	 */
	DocumentContentData getDocumentContent(String id) throws Exception;

	/**
	 * Gets the document list.
	 *
	 * @param documentType
	 *            the document type
	 * @param maxNumberPages
	 *            the max number pages
	 * @return the document list
	 * @throws Exception
	 *             the exception
	 */
	List<DocumentElement> getDocumentList(DocumentType documentType, int maxNumberPages) throws Exception;



	/**
	 * Gets the document list.
	 *
	 * @param documentType
	 *            the document type
	 * @param year
	 *            the year
	 * @param maxNumberPages
	 *            the max number pages
	 * @return the document list
	 * @throws Exception
	 *             the exception
	 */
	List<DocumentElement> getDocumentList(DocumentType documentType,Integer year, int maxNumberPages) throws Exception;

	/**
	 * Gets the document list.
	 *
	 * @param year
	 *            the year
	 * @param maxNumberPages
	 *            the max number pages
	 * @return the document list
	 * @throws Exception
	 *             the exception
	 */
	List<DocumentElement> getDocumentList(Integer year, int maxNumberPages) throws Exception;

	/**
	 * Gets the document list.
	 *
	 * @param changedSinceDate
	 *            the changed since date
	 * @param maxNumberPages
	 *            the max number pages
	 * @return the document list
	 * @throws Exception
	 *             the exception
	 */
	List<DocumentElement> getDocumentList(String changedSinceDate, int maxNumberPages) throws Exception;

	/**
	 * Gets the document list by person.
	 *
	 * @param personId
	 *            the person id
	 * @param maxNumberPages
	 *            the max number pages
	 * @return the document list by person
	 * @throws Exception
	 *             the exception
	 */
	List<DocumentElement> getDocumentListByPerson(String personId, int maxNumberPages) throws Exception;


	/**
	 * Gets the document status.
	 *
	 * @param id
	 *            the id
	 * @return the document status
	 * @throws Exception
	 *             the exception
	 */
	DocumentStatusContainer getDocumentStatus(String id) throws Exception;

	/**
	 * Gets the person.
	 *
	 * @param id
	 *            the id
	 * @return the person
	 * @throws Exception
	 *             the exception
	 */
	PersonData getPerson(String id) throws Exception;


	/**
	 * Gets the person list.
	 *
	 * @return the person list
	 * @throws Exception
	 *             the exception
	 */
	PersonContainerElement getPersonList() throws Exception;

	/**
	 * Process document list.
	 *
	 * @param documentType
	 *            the document type
	 * @param year
	 *            the year
	 * @param processStrategy
	 *            the process strategy
	 * @throws Exception
	 *             the exception
	 */
	void processDocumentList(DocumentType documentType,Integer year,ProcessDataStrategy<DocumentElement> processStrategy) throws Exception;




	/**
	 * Process document list.
	 *
	 * @param documentType
	 *            the document type
	 * @param processStrategy
	 *            the process strategy
	 * @throws Exception
	 *             the exception
	 */
	void processDocumentList(DocumentType documentType,ProcessDataStrategy<DocumentElement> processStrategy) throws Exception;

	/**
	 * Process document list.
	 *
	 * @param year
	 *            the year
	 * @param processStrategy
	 *            the process strategy
	 * @throws Exception
	 *             the exception
	 */
	void processDocumentList(Integer year,ProcessDataStrategy<DocumentElement> processStrategy) throws Exception;

	/**
	 * Process document list.
	 *
	 * @param changedSinceDate
	 *            the changed since date
	 * @param processStrategy
	 *            the process strategy
	 * @throws Exception
	 *             the exception
	 */
	void processDocumentList(String changedSinceDate, ProcessDataStrategy<DocumentElement> processStrategy) throws Exception;

	/**
	 * Process document list by person.
	 *
	 * @param personId
	 *            the person id
	 * @param processStrategy
	 *            the process strategy
	 * @throws Exception
	 *             the exception
	 */
	void processDocumentListByPerson(String personId, ProcessDataStrategy<DocumentElement> processStrategy) throws Exception;

	/**
	 * Gets the ballot list.
	 *
	 * @return the ballot list
	 * @throws Exception
	 *             the exception
	 */
	List<BallotDocumentElement> getBallotList() throws Exception;

}
