/*
 * Copyright 2010 James Pether Sörling
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *	$Id: RiksdagenApiImpl.java 4642 2010-09-24 10:45:33Z pether $
 *  $HeadURL: https://cia.svn.sourceforge.net/svnroot/cia/tags/service.external.riksdagen-1.0.0-beta-1.1/src/main/java/com/hack23/cia/service/external/riksdagen/impl/RiksdagenApiImpl.java $
*/
package com.hack23.cia.service.external.riksdagen.impl;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBElement;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.oxm.Unmarshaller;
import org.springframework.stereotype.Component;

import com.hack23.cia.model.external.riksdagen.documentcontent.impl.DocumentContentData;
import com.hack23.cia.model.external.riksdagen.dokumentlista.impl.DocumentContainerElement;
import com.hack23.cia.model.external.riksdagen.dokumentlista.impl.DocumentElement;
import com.hack23.cia.model.external.riksdagen.dokumentstatus.impl.DocumentStatusContainer;
import com.hack23.cia.model.external.riksdagen.dokumentstatus.impl.DocumentType;
import com.hack23.cia.model.external.riksdagen.person.impl.PersonData;
import com.hack23.cia.model.external.riksdagen.personlista.impl.PersonContainerElement;
import com.hack23.cia.model.external.riksdagen.utskottsforslag.impl.CommitteeProposalComponentData;
import com.hack23.cia.model.external.riksdagen.votering.impl.BallotContainer;
import com.hack23.cia.model.external.riksdagen.votering.impl.VoteData;
import com.hack23.cia.model.external.riksdagen.voteringlista.impl.BallotDocumentElement;
import com.hack23.cia.model.external.riksdagen.voteringlista.impl.VoteListContainerElement;

import com.hack23.cia.service.external.common.api.ProcessDataStrategy;
import com.hack23.cia.service.external.common.impl.AbstractXmlAgentImpl;
import com.hack23.cia.service.external.riksdagen.api.RiksdagenApi;

/**
 * The Class RiksdagenApiImpl.
 */
@Component
@SuppressWarnings("unchecked")
public class RiksdagenApiImpl extends AbstractXmlAgentImpl implements
		RiksdagenApi {

	/** The BALLOT. */
	private static String BALLOT = "http://data.riksdagen.se/votering/${ID_KEY}/xml";

	/** The COMMITT e_ proposal. */
	private static String COMMITTE_PROPOSAL = "http://data.riksdagen.se/utskottsforslag/${ID_KEY}/xml";

	/** The BALLO t_ list. */
	private static String BALLOT_LIST="http://data.riksdagen.se/voteringlista/?rm=&bet=&punkt=&iid=&parti=&valkrets=&rost=&sz=10000&utformat=xml&gruppering=votering_id";

	/** The DOCUMEN t_ content. */
	private static String DOCUMENT_CONTENT = "http://data.riksdagen.se/dokument/${DOC_ID}/text";

	/** The DOCUMEN t_ lis t_ change d_ date. */
	private static String DOCUMENT_LIST_CHANGED_DATE = "http://data.riksdagen.se/dokumentlista/?rm=&typ=&d=${CHANGED_SINCE}&ts=&parti=&iid=&bet=&org=&kat=&sz=200&sort=c&utformat=xml";

	/** The DOCUMEN t_ lis t_ person. */
	private static String DOCUMENT_LIST_PERSON = "http://data.riksdagen.se/dokumentlista/?rm=&typ=&d=&ts=&parti=&iid=${PERSON_ID}&bet=&org=&kat=&sz=200&sort=c&utformat=xml";

	/** The DOCUMEN t_ lis t_ type. */
	private static String DOCUMENT_LIST_TYPE = "http://data.riksdagen.se/dokumentlista/?rm=&typ=${TYPE}&d=&ts=&parti=&iid=&bet=&org=&kat=&sz=200&sort=c&utformat=xml";

	/** The DOCUMEN t_ lis t_ typ e_ year. */
	private static String DOCUMENT_LIST_TYPE_YEAR = "http://data.riksdagen.se/dokumentlista/?rm=${YEAR}&typ=${TYPE}&d=&ts=&parti=&iid=&bet=&org=&kat=&sz=200&sort=c&utformat=xml";

	/** The DOCUMEN t_ lis t_ year. */
	private static String DOCUMENT_LIST_YEAR = "http://data.riksdagen.se/dokumentlista/?rm=${YEAR}&typ=&d=&ts=&parti=&iid=&bet=&org=&kat=&sz=200&sort=c&utformat=xml";

	/** The DOCUMEN t_ status. */
	private static String DOCUMENT_STATUS = "http://data.riksdagen.se/dokumentstatus/${ID_KEY}/xml";

	/** The logger. */
	private static Logger logger = LoggerFactory
			.getLogger(RiksdagenApiImpl.class);

	/** The PERSON. */
	private static String PERSON = "http://data.riksdagen.se/person/${ID_KEY}";

	/** The PERSO n_ list. */
	private static String PERSON_LIST = "http://data.riksdagen.se/personlista/?iid=&fnamn=&enamn=&f_ar=&kn=&parti=&valkrets=&rdlstatus=samtliga&org=";

	/** The person list unmarshaller. */
	@Autowired
	@Qualifier("riksdagenPersonListMarshaller")
	private Unmarshaller personListUnmarshaller;

	/** The person unmarshaller. */
	@Autowired
	@Qualifier("riksdagenPersonMarshaller")
	private Unmarshaller personUnmarshaller;

	/** The riksdagen ballot marshaller. */
	@Autowired
	@Qualifier("riksdagenBallotMarshaller")
	private Unmarshaller riksdagenBallotMarshaller;

	/** The riksdagen ballot list marshaller. */
	@Autowired
	@Qualifier("riksdagenBallotListMarshaller")
	private Unmarshaller riksdagenBallotListMarshaller;


	/** The riksdagen committee proposal marshaller. */
	@Autowired
	@Qualifier("riksdagenCommitteeProposalMarshaller")
	private Unmarshaller riksdagenCommitteeProposalMarshaller;

	/** The riksdagen document list marshaller. */
	@Autowired
	@Qualifier("riksdagenDocumentListMarshaller")
	private Unmarshaller riksdagenDocumentListMarshaller;

	/** The riksdagen document status marshaller. */
	@Autowired
	@Qualifier("riksdagenDocumentStatusMarshaller")
	private Unmarshaller riksdagenDocumentStatusMarshaller;

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * com.hack23.cia.service.component.agent.impl.riksdagen.api.RiksdagenApi
	 * #getBallot(java.lang.String)
	 */
	@Override
	public List<VoteData> getBallot(final String id) throws Exception {
		final String url = BALLOT.replace("${ID_KEY}", id);
		return ((JAXBElement<BallotContainer>) this.unmarshallXml(
				riksdagenBallotMarshaller, url,
				"http://votering.riksdagen.external.model.cia.hack23.com/impl",null,null))
				.getValue().getBallotDocumentData().getVoteDataList();
	}

	/* (non-Javadoc)
	 * @see com.hack23.cia.service.external.riksdagen.api.RiksdagenApi#getBallotList()
	 */
	@Override
	public List<BallotDocumentElement> getBallotList() throws Exception {
		return ((JAXBElement<VoteListContainerElement>) this.unmarshallXml(
				riksdagenBallotListMarshaller, BALLOT_LIST,
				"http://voteringlista.riksdagen.external.model.cia.hack23.com/impl",null,null))
				.getValue().getVotering();
	}


	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * com.hack23.cia.service.component.agent.impl.riksdagen.api.RiksdagenApi
	 * #getCommitteeProposal(java.lang.String)
	 */
	@Override
	public CommitteeProposalComponentData getCommitteeProposal(final String id)
			throws Exception {
		final String url = COMMITTE_PROPOSAL.replace("${ID_KEY}", id);
		return ((JAXBElement<CommitteeProposalComponentData>) this
				.unmarshallXml(riksdagenCommitteeProposalMarshaller, url,
						"http://utskottsforslag.riksdagen.external.model.cia.hack23.com/impl",null,null))
				.getValue();
	}

	/* (non-Javadoc)
	 * @see com.hack23.cia.service.external.riksdagen.api.RiksdagenApi#getDocumentContent(java.lang.String)
	 */
	@Override
	public DocumentContentData getDocumentContent(final String id) throws Exception {
		final String url = DOCUMENT_CONTENT.replace("${DOC_ID}", id);
		final DocumentContentData documentContentData = new DocumentContentData();
		documentContentData.setId(id);

		final String content = this.retriveContent(url);
		documentContentData.setContent(content);
		return documentContentData;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * com.hack23.cia.service.external.riksdagen.api.RiksdagenApi#getDocumentList
	 * (com.hack23.cia.service.external.riksdagen.api.RiksdagenApi.DocumentType)
	 */
	@Override
	public List<DocumentElement> getDocumentList(final DocumentType documentType,
			final int maxNumberPages) throws Exception {
		return loadDocumentList(
				DOCUMENT_LIST_TYPE.replace("${TYPE}", documentType.value()),
				maxNumberPages);
	}



	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * com.hack23.cia.service.external.riksdagen.api.RiksdagenApi#getDocumentList
	 * (com.hack23.cia.service.external.riksdagen.api.RiksdagenApi.DocumentType,
	 * java.lang.Integer)
	 */
	@Override
	public List<DocumentElement> getDocumentList(final DocumentType documentType,
			final Integer year, final int maxNumberPages) throws Exception {
		return loadDocumentList(
				DOCUMENT_LIST_TYPE_YEAR
						.replace("${TYPE}", documentType.value()).replace(
								"${YEAR}", year.toString()), maxNumberPages);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * com.hack23.cia.service.external.riksdagen.api.RiksdagenApi#getDocumentList
	 * (java.lang.Integer)
	 */
	@Override
	public List<DocumentElement> getDocumentList(final Integer year,
			final int maxNumberPages) throws Exception {
		return loadDocumentList(
				DOCUMENT_LIST_YEAR.replace("${YEAR}", year.toString()),
				maxNumberPages);
	}


	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * com.hack23.cia.service.external.riksdagen.api.RiksdagenApi#getDocumentList
	 * (java.lang.String)
	 */
	@Override
	public List<DocumentElement> getDocumentList(final String changedSinceDate,
			final int maxNumberPages) throws Exception {
		return loadDocumentList(DOCUMENT_LIST_CHANGED_DATE.replace(
				"${CHANGED_SINCE}", changedSinceDate), maxNumberPages);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.hack23.cia.service.external.riksdagen.api.RiksdagenApi#
	 * getDocumentListByPerson(java.lang.String)
	 */
	@Override
	public List<DocumentElement> getDocumentListByPerson(final String personId,
			final int maxNumberPages) throws Exception {
		return loadDocumentList(
				DOCUMENT_LIST_PERSON.replace("${PERSON_ID}", personId),
				maxNumberPages);
	}


	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * com.hack23.cia.service.component.agent.impl.riksdagen.api.RiksdagenApi
	 * #getDocumentStatus(java.lang.String)
	 */
	@Override
	public DocumentStatusContainer getDocumentStatus(final String id)
			throws Exception {
		final String url = DOCUMENT_STATUS.replace("${ID_KEY}", id);

		return ((JAXBElement<DocumentStatusContainer>) this
				.unmarshallXml(riksdagenDocumentStatusMarshaller, url,
						"http://dokumentstatus.riksdagen.external.model.cia.hack23.com/impl",null,null))
				.getValue();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * com.hack23.cia.service.component.agent.impl.riksdagen.api.RiksdagenApi
	 * #getPerson(java.lang.String)
	 */
	@Override
	public PersonData getPerson(final String id) throws Exception {
		final String url = PERSON.replace("${ID_KEY}", id);
		return ((JAXBElement<com.hack23.cia.model.external.riksdagen.person.impl.PersonContainerData>) this
				.unmarshallXml(personUnmarshaller, url,
						"http://person.riksdagen.external.model.cia.hack23.com/impl",null,null))
				.getValue().getPerson();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * com.hack23.cia.service.component.agent.impl.riksdagen.api.RiksdagenApi
	 * #getPersonList()
	 */
	@Override
	public PersonContainerElement getPersonList() throws Exception {
		return ((JAXBElement<PersonContainerElement>) this
				.unmarshallXml(personListUnmarshaller, PERSON_LIST,
						"http://personlista.riksdagen.external.model.cia.hack23.com/impl",null,null))
				.getValue();
	}

	/**
	 * Load and process document list.
	 *
	 * @param url
	 *            the url
	 * @param processStrategy
	 *            the process strategy
	 * @throws Exception
	 *             the exception
	 */
	private void LoadAndProcessDocumentList(final String url,
			final ProcessDataStrategy<DocumentElement> processStrategy)
			throws Exception {
		DocumentContainerElement dokumentLista = ((JAXBElement<DocumentContainerElement>) this
				.unmarshallXml(riksdagenDocumentListMarshaller, url,
						"http://dokumentlista.riksdagen.external.model.cia.hack23.com/impl",null,null))
				.getValue();

		int resultSize = dokumentLista.getDokument().size();
		processAll(dokumentLista.getDokument(), processStrategy);
		final BigInteger pages = dokumentLista.getTotalPages();
		for (int i = 1; ((i < pages.intValue())); i++) {
			dokumentLista = ((JAXBElement<DocumentContainerElement>) this
					.unmarshallXml(riksdagenDocumentListMarshaller,
							dokumentLista.getNextPage(),
							"http://dokumentlista.riksdagen.external.model.cia.hack23.com/impl",null,null))
					.getValue();
			resultSize = resultSize + dokumentLista.getDokument().size();
			processAll(dokumentLista.getDokument(), processStrategy);
			logger.info("Loading documents: " + resultSize + "/"
					+ dokumentLista.getHits());
		}
	}

	/**
	 * Load document list.
	 *
	 * @param url
	 *            the url
	 * @param maxNumberPages
	 *            the max number pages
	 * @return the list
	 * @throws Exception
	 *             the exception
	 */
	private List<DocumentElement> loadDocumentList(final String url,
			final int maxNumberPages) throws Exception {
		final List<DocumentElement> result = new ArrayList<DocumentElement>();

		DocumentContainerElement dokumentLista = ((JAXBElement<DocumentContainerElement>) this
				.unmarshallXml(riksdagenDocumentListMarshaller, url,
						"http://dokumentlista.riksdagen.external.model.cia.hack23.com/impl",null,null))
				.getValue();
		result.addAll(dokumentLista.getDokument());
		final BigInteger pages = dokumentLista.getTotalPages();
		for (int i = 1; ((i < pages.intValue()) && (i < maxNumberPages)); i++) {
			dokumentLista = ((JAXBElement<DocumentContainerElement>) this
					.unmarshallXml(riksdagenDocumentListMarshaller,
							dokumentLista.getNextPage(),
							"http://dokumentlista.riksdagen.external.model.cia.hack23.com/impl",null,null))
					.getValue();
			result.addAll(dokumentLista.getDokument());
			logger.info("Loading documents: " + result.size() + "/"
					+ dokumentLista.getHits());
		}

		return result;
	}

	/**
	 * Process all.
	 *
	 * @param dokument
	 *            the dokument
	 * @param processStrategy
	 *            the process strategy
	 */
	private void processAll(final List<DocumentElement> dokument,
			final ProcessDataStrategy<DocumentElement> processStrategy) {
		for (final DocumentElement documentElement : dokument) {

			try {
			processStrategy.process(documentElement);
			} catch (final Exception e) {
				logger.warn("Error processing document :" + documentElement.getId(),e);
			}
		}
	}

	/* (non-Javadoc)
	 * @see com.hack23.cia.service.external.riksdagen.api.RiksdagenApi#processDocumentList(com.hack23.cia.model.external.riksdagen.dokumentstatus.impl.DocumentType, java.lang.Integer, com.hack23.cia.service.external.common.api.ProcessDataStrategy)
	 */
	@Override
	public void processDocumentList(final DocumentType documentType,
			final Integer year,final ProcessDataStrategy<DocumentElement> processStrategy) throws Exception {
		LoadAndProcessDocumentList(
				DOCUMENT_LIST_TYPE_YEAR
						.replace("${TYPE}", documentType.value()).replace(
								"${YEAR}", year.toString()), processStrategy);
	}

	/* (non-Javadoc)
	 * @see com.hack23.cia.service.external.riksdagen.api.RiksdagenApi#processDocumentList(com.hack23.cia.model.external.riksdagen.dokumentstatus.impl.DocumentType, com.hack23.cia.service.external.common.api.ProcessDataStrategy)
	 */
	@Override
	public void processDocumentList(final DocumentType documentType,
			final ProcessDataStrategy<DocumentElement> processStrategy) throws Exception {
		LoadAndProcessDocumentList(
				DOCUMENT_LIST_TYPE.replace("${TYPE}", documentType.value()),
				processStrategy);
	}

	/* (non-Javadoc)
	 * @see com.hack23.cia.service.external.riksdagen.api.RiksdagenApi#processDocumentList(java.lang.Integer, com.hack23.cia.service.external.common.api.ProcessDataStrategy)
	 */
	@Override
	public void processDocumentList(final Integer year,
			final ProcessDataStrategy<DocumentElement> processStrategy) throws Exception {
		LoadAndProcessDocumentList(
				DOCUMENT_LIST_YEAR.replace("${YEAR}", year.toString()),
				processStrategy);
	}

	/* (non-Javadoc)
	 * @see com.hack23.cia.service.external.riksdagen.api.RiksdagenApi#processDocumentList(java.lang.String, com.hack23.cia.service.external.common.api.ProcessDataStrategy)
	 */
	@Override
	public void processDocumentList(final String changedSinceDate,
			final ProcessDataStrategy<DocumentElement> processStrategy) throws Exception {
		LoadAndProcessDocumentList(DOCUMENT_LIST_CHANGED_DATE.replace(
				"${CHANGED_SINCE}", changedSinceDate), processStrategy);
	}

	/* (non-Javadoc)
	 * @see com.hack23.cia.service.external.riksdagen.api.RiksdagenApi#processDocumentListByPerson(java.lang.String, com.hack23.cia.service.external.common.api.ProcessDataStrategy)
	 */
	@Override
	public void processDocumentListByPerson(final String personId,final ProcessDataStrategy<DocumentElement> processStrategy) throws Exception {
		 LoadAndProcessDocumentList(
				DOCUMENT_LIST_PERSON.replace("${PERSON_ID}", personId),
				processStrategy);
	}

}
